@@ -106,6 +106,23 @@ void write_state_file (struct tgl_state *TLS) {
   close (state_file_fd); 
 }
 
+static gboolean write_state_file_gw (gpointer data) {
+  struct tgl_state *TLS = data;
+  
+  ((connection_data *)TLS->ev_base)->write_timer = 0;
+  write_state_file ((struct tgl_state *)TLS);
+  
+  return FALSE;
+}
+
+void write_state_file_schedule (struct tgl_state *TLS) {
+  connection_data *conn = TLS->ev_base;
+  
+  if (! conn->write_timer) {
+    conn->write_timer = purple_timeout_add (0, write_state_file_gw, TLS);
+  }
+}
+
 void write_dc (struct tgl_dc *DC, void *extra) {
   int auth_file_fd = *(int *)extra;
   if (!DC) { 
@@ -161,17 +178,19 @@ void read_dc (struct tgl_state *TLS, int auth_file_fd, int id, unsigned ver) {
   assert (read (auth_file_fd, &auth_key_id, 8) == 8);
   assert (read (auth_file_fd, auth_key, 256) == 256);
 
-  //bl_do_add_dc (id, ip, l, port, auth_key_id, auth_key);
   bl_do_dc_option (TLS, id, 2, "DC", l, ip, port);
   bl_do_set_auth_key_id (TLS, id, auth_key);
   bl_do_dc_signed (TLS, id);
 }
 
-int error_if_val_false (struct tgl_state *TLS, int val, const char *msg) {
+int error_if_val_false (struct tgl_state *TLS, int val, const char *cause, const char *msg) {
   if (!val) {
     connection_data *conn = TLS->ev_base;
     purple_connection_error_reason (conn->gc, PURPLE_CONNECTION_ERROR_AUTHENTICATION_FAILED, msg);
-    return 1;
+    
+    purple_notify_message (_telegram_protocol, PURPLE_NOTIFY_MSG_ERROR,
+            cause, msg, NULL, NULL, NULL);
+    return TRUE;
   }
   return 0;
 }
@@ -247,7 +266,6 @@ void write_secret_chat (tgl_peer_t *_P, void *extra) {
   
   int id = tgl_get_peer_id (P->id);
   assert (write (fd, &id, 4) == 4);
-  //assert (write (fd, &P->flags, 4) == 4);
   int l = strlen (P->print_name);
   assert (write (fd, &l, 4) == 4);
   assert (write (fd, P->print_name, l) == l);
@@ -298,7 +316,6 @@ void read_secret_chat (struct tgl_state *TLS, int fd, int v) {
   static unsigned char key[256];
   static unsigned char sha[20];
   assert (read (fd, &id, 4) == 4);
-  //assert (read (fd, &flags, 4) == 4);
   assert (read (fd, &l, 4) == 4);
   assert (l > 0 && l < 1000);
   assert (read (fd, s, l) == l);
@@ -368,7 +385,7 @@ void read_secret_chat_file (struct tgl_state *TLS) {
 
 void telegram_export_authorization (struct tgl_state *TLS);
 void export_auth_callback (struct tgl_state *TLS, void *extra, int success) {
-  if (!error_if_val_false(TLS, success, "Authentication Export failed.")) {
+  if (!error_if_val_false(TLS, success, "Login Canceled", "Authentication export failed.")) {
     telegram_export_authorization (TLS);
   }
 }
@@ -376,6 +393,7 @@ void export_auth_callback (struct tgl_state *TLS, void *extra, int success) {
 void telegram_export_authorization (struct tgl_state *TLS) {
   int i;
   for (i = 0; i <= TLS->max_dc_num; i++) if (TLS->DC_list[i] && !tgl_signed_dc (TLS, TLS->DC_list[i])) {
+    debug ("tgl_do_export_auth(%d)", i);
     tgl_do_export_auth (TLS, i, export_auth_callback, (void*)(long)TLS->DC_list[i]);    
     return;
   }
@@ -387,7 +405,7 @@ static void request_code (struct tgl_state *TLS);
 static void request_name_and_code (struct tgl_state *TLS);
 static void code_receive_result (struct tgl_state *TLS, void *extra, int success, struct tgl_user *U) {
   if (!success) {
-    debug ("Bad code...\n");
+    debug ("Bad code...");
     request_code (TLS);
   } else {
     telegram_export_authorization (TLS);
@@ -396,7 +414,7 @@ static void code_receive_result (struct tgl_state *TLS, void *extra, int success
 
 static void code_auth_receive_result (struct tgl_state *TLS, void *extra, int success, struct tgl_user *U) {
   if (!success) {
-    debug ("Bad code...\n");
+    debug ("Bad code...");
     request_name_and_code (TLS);
   } else {
     telegram_export_authorization (TLS);
@@ -435,7 +453,7 @@ static void request_name_code_entered (PurpleConnection* gc, PurpleRequestFields
 }
 
 static void request_code (struct tgl_state *TLS) {
-  debug ("Client is not registered, registering...\n");
+  debug ("Client is not registered, registering...");
   connection_data *conn = TLS->ev_base;
   int compat = purple_account_get_bool (tg_get_acc(TLS), "compat-verification", 0);
   
@@ -456,7 +474,7 @@ static void request_code (struct tgl_state *TLS) {
 }
 
 static void request_name_and_code (struct tgl_state *TLS) {
-  debug ("Phone is not registered, registering...\n");
+  debug ("Phone is not registered, registering...");
 
   connection_data *conn = TLS->ev_base;
 
@@ -486,7 +504,10 @@ static void request_name_and_code (struct tgl_state *TLS) {
 
 static void sign_in_callback (struct tgl_state *TLS, void *extra, int success, int registered, const char *mhash) {
   connection_data *conn = TLS->ev_base;
-  if (!error_if_val_false (TLS, success, "Invalid or non-existing phone number.")) {
+  if (!error_if_val_false (TLS, success, "Invalid phone number",
+          "Please enter only numbers in the international phone number format, "
+          "a leading + following by the country prefix and the phone number.\n"
+          "Do not use any other special chars.")) {
     conn->hash = strdup (mhash);
     if (registered) {
       request_code (TLS);
@@ -518,7 +539,9 @@ static int all_authorized (struct tgl_state *TLS) {
 
 static int check_all_authorized (gpointer arg) {
   struct tgl_state *TLS = arg;
+
   if (all_authorized (TLS)) {
+    ((connection_data *)TLS->ev_base)->login_timer = 0;
     telegram_send_sms (TLS);    
     return FALSE;
   } else {
@@ -526,7 +549,9 @@ static int check_all_authorized (gpointer arg) {
   }
 }
     
-void telegram_login (struct tgl_state *TLS) {    
+void telegram_login (struct tgl_state *TLS) {
+  connection_data *conn = TLS->ev_base;
+  
   read_auth_file (TLS);
   read_state_file (TLS);
   read_secret_chat_file (TLS);
@@ -534,66 +559,13 @@ void telegram_login (struct tgl_state *TLS) {
     telegram_send_sms (TLS);
     return;
   }
-  purple_timeout_add (100, check_all_authorized, TLS);
-}
-
-PurpleConversation *chat_show (PurpleConnection *gc, int id) {
-  debug ("show chat");
-  connection_data *conn = purple_connection_get_protocol_data(gc);
-  
-  PurpleConversation *convo = purple_find_chat(gc, id);
-  if (! convo) {
-    gchar *name = g_strdup_printf ("%d", id);
-    if (! g_hash_table_lookup (conn->joining_chats, name)) {
-      g_hash_table_insert(conn->joining_chats, name, 0);
-      tgl_do_get_chat_info (conn->TLS, TGL_MK_CHAT(id), 0, on_chat_get_info, NULL);
-    } else {
-      g_free(name);
-    }
-  }
-  return convo;
-}
-
-int chat_add_message (struct tgl_state *TLS, struct tgl_message *M, char *text) {
-  connection_data *conn = TLS->ev_base;
-  
-  if (chat_show (conn->gc, tgl_get_peer_id (M->to_id))) {
-    p2tgl_got_chat_in(TLS, M->to_id, M->from_id, text ? text : M->message,
-                      M->service ? PURPLE_MESSAGE_SYSTEM : PURPLE_MESSAGE_RECV, M->date);
-    
-    pending_reads_add (conn->pending_reads, M->to_id);
-    if (p2tgl_status_is_present(purple_account_get_active_status(conn->pa))) {
-      pending_reads_send_all (conn->pending_reads, conn->TLS);
-    }
-    return 1;
-  } else {
-    // add message once the chat was initialised
-    struct message_text *mt = message_text_init (M, text);
-    g_queue_push_tail (conn->new_messages, mt);
-    return 0;
-  }
-}
-
-void chat_add_all_users (PurpleConversation *pc, struct tgl_chat *chat) {
-  struct tgl_chat_user *curr =  chat->user_list;
-  if (!curr) {
-    warning ("add_all_users_to_chat: chat contains no user list, cannot add users\n.");
-    return;
-  }
-  
-  int i;
-  for (i = 0; i < chat->user_list_size; i++) {
-    struct tgl_chat_user *uid = (curr + i);
-    int flags = (chat->admin_id == uid->user_id ? PURPLE_CBFLAGS_FOUNDER : PURPLE_CBFLAGS_NONE);
-    p2tgl_conv_add_user(pc, *uid, NULL, flags, 0);
-  }
+  conn->login_timer = purple_timeout_add (100, check_all_authorized, TLS);
 }
 
 /**
  * This function generates a png image to visualize the sha1 key from an encrypted chat.
  */
-int generate_ident_icon (struct tgl_state *TLS, unsigned char* sha1_key)
-{
+int generate_ident_icon (struct tgl_state *TLS, unsigned char* sha1_key) {
   int colors[4] = {
     0xffffff,
     0xd5e6f3,
